'use strict';

/* eslint-disable prettier/prettier */
/* eslint-disable no-useless-escape */
/**
  Motivation:
    - Testing emails with dynamic patterns for the email domain and username.
  Details:
    - For the username:
      * Only alphanumeric characters and the set of characters . - _ will be accepted.
      * It must start with an alphanumeric character, and after that, it can be preceded by . - _
      * There can't be sequences of . - _; between each character, there must be at least 1 alphanumeric character.
      * The end of the email name cannot end with . - _
    - For the email domain:
      * Domains with a minimum of 2 characters and a maximum of 26 characters will be accepted (according to the domain registration rules at
        https://registro.br/dominio/regras/). If there is a need for minimum and/or maximum values for the domain, you should use the
        @param{paramsEmailValidator.minMaxEmailDomain}.
      * It may contain characters of type . - _, but there can't be sequences of these characters.
      * If there is a need to validate a specific domain or with specific regex rules, you can use the @param{paramsEmailValidator.emailDomainName} typeof RegExp.

    Example:
      const emailIsValid = emailValidator({ email: 'jon.doe@hotmail.com', paramsEmailValidator: {
        emailDomainName: RegExp(/hotmail/)',
        minMaxUserNameEmail: {max: 15}
      }});
      console.log('is it a valid email?: ', emailIsValid) // true
      * Atention:
    - When passing arguments that will be used in the size rules of the username or domain, the expected value should be passed decreased by 1 unit.
      ** Exemple:
        The minimum size of the username should be 10 characters, and the maximum size should be 15 characters. In @param{paramsEmailValidator.minMaxUserNameEmail},
        the argument to be used is: { min: 9, max: 14 }.
*/
const emailValidator = ({ email, paramsEmailValidator, }) => {
    if (!email)
        return false;
    const patternValidator = `^(([a-zA-Z0-9][\-_.]{0,1})){${paramsEmailValidator?.minMaxUserNameEmail?.min ?? 1},${paramsEmailValidator?.minMaxUserNameEmail?.max ?? ''}}([a-zA-Z0-9]@)${paramsEmailValidator?.emailDomainName?.source ?? `(([a-zA-Z0-9][-.]{0,1}){${paramsEmailValidator?.minMaxEmailDomain?.min ?? 2},${paramsEmailValidator?.minMaxEmailDomain?.max ?? 26}})([a-zA-Z0-9][.]{1}[a-zA-Z0-9]+)$`}`;
    const regexValidator = new RegExp(patternValidator);
    return regexValidator.test(email);
};

const normalizeToValidation = ({ value, length, canRepeatNumber = false, }) => {
    if (!value)
        return undefined;
    value = value.toString()?.replace(/\D/g, '');
    if (length) {
        if (value.length !== length)
            return undefined;
    }
    if (!canRepeatNumber) {
        const invalidSequences = new Set(Array.from(Array(10).keys()).map((f) => String(f).repeat(length ?? 0)));
        if (invalidSequences.has(value))
            return undefined;
    }
    return value;
};

const cnpjValidator = (cnpj) => {
    const VALID_LENGTH = 14;
    cnpj = normalizeToValidation({ value: cnpj, length: VALID_LENGTH });
    if (!cnpj)
        return false;
    let sum = 0;
    let factor = 5;
    for (let i = 0; i < 12; i++) {
        sum += parseInt(cnpj.charAt(i), 10) * factor;
        factor = factor === 2 ? 9 : factor - 1;
    }
    let result = sum % 11 < 2 ? 0 : 11 - (sum % 11);
    if (result !== parseInt(cnpj.charAt(12), 10)) {
        return false;
    }
    sum = 0;
    factor = 6;
    for (let i = 0; i < 13; i++) {
        sum += parseInt(cnpj.charAt(i), 10) * factor;
        factor = factor === 2 ? 9 : factor - 1;
    }
    result = sum % 11 < 2 ? 0 : 11 - (sum % 11);
    if (result !== parseInt(cnpj.charAt(13), 10))
        return false;
    return true;
};

const nisValidator = (nis) => {
    const VALID_LENGTH = 11;
    nis = normalizeToValidation({ value: nis, length: VALID_LENGTH });
    if (!nis)
        return false;
    const multiplierBase = 3298765432;
    let total = 0;
    for (let i = 0; i < 10; i++) {
        const multiplying = Number(nis[i]);
        const multiplier = Number(String(multiplierBase).charAt(i));
        total += multiplying * multiplier;
    }
    let remainder = 11 - (total % 11);
    remainder = remainder === 10 || remainder === 11 ? 0 : remainder;
    const digit = Number(nis.charAt(10));
    return remainder === digit;
};

const cpfValidator = (cpf) => {
    const VALID_LENGTH = 11;
    cpf = normalizeToValidation({ value: cpf, length: VALID_LENGTH });
    if (!cpf)
        return false;
    let sum = 0;
    for (let i = 1; i <= 9; i++)
        sum += parseInt(cpf[i - 1], 10) * (11 - i);
    let remainder = (sum * 10) % 11;
    if (remainder === 10 || remainder === 11)
        remainder = 0;
    if (remainder !== parseInt(cpf.substring(9, 10), 10))
        return false;
    sum = 0;
    for (let j = 1; j <= 10; j++)
        sum += parseInt(cpf[j - 1], 10) * (12 - j);
    remainder = (sum * 10) % 11;
    if (remainder === 10 || remainder === 11)
        remainder = 0;
    if (remainder !== parseInt(cpf[10], 10))
        return false;
    return true;
};

/**
  Motivation:
    - Phone number test with validations of mobile phone numbers (BR), public utility numbers (BR),
      numbers without area code (BR), international numbers and custom validations with regex patterns.
  Details:
    - Numbering structure of a telephone number:
        CC = Coutry Code;
        NDC = Number Area State Code (National Destination Code);
        N = Number phone;
        +CC NDC N => +55 85 99999-9999
        reference: https://www.itu.int/rec/T-REC-E.164-201011-I/en

    - Validation priority order:
      * If there is @param{paramsPhoneValidator}, the following order is parsed:
        1º: If there is @param{param Phone Validator.custom Pattern Phone};
        2º: Whether validation should occur with international phone numbers, based on E.164;
        3º: Whether validation should only occur with mobile phone numbers (BR);
            - If @param{paramsPhoneValidator.onlyMobilePhoneBR} is true, there is a condition that
              validation occurs with or without the DDD digits, with @param{paramsPhoneValidator.numberWithoutCodeAreas}.
        4º: Whether validation should occur only for public utility numbers;
      * If there is no @param{paramsPhoneValidator} or if you have @param{paramsPhoneValidator.numberWithoutCodeAreas},
            validation will occur with standard Brazilian numbers.

    - Rules for telephone numbering in Brazil:
      Cellular Telephony - Initial digit:
        - 9 for Bands A (96 to 99) and B (91 to 94);
        - 8 for Bands D and E;
        - 7 Cellular and Trunking (Nextel);
        - 6 for bands A, B, D and E*;
      Fixed Telephony - Initial digit
        - 2 to 5;
      Public utility numbers:
        - Composed of 3 to 4 numbers, starting with the digit 1.
      reference: https://www.teleco.com.br/num.asp
 */
const phoneValidator = ({ phone, paramsPhoneValidator, }) => {
    phone = normalizeToValidation({ value: phone });
    if (!phone)
        return false;
    let numbersWithouCodeAreas = false;
    if (paramsPhoneValidator) {
        const { onlyMobilePhoneBR, canReceiveInternationalNumbers, customPatternPhone, publicUtilityNumberBR, } = paramsPhoneValidator;
        if (customPatternPhone &&
            Object.values(customPatternPhone).every((item) => item)) {
            const { coutryCode, areaStateCode, phoneNumber } = customPatternPhone;
            const regexCustomPatternPhone = new RegExp(`${coutryCode.source}${areaStateCode.source}${phoneNumber.source}`);
            try {
                return Boolean(regexCustomPatternPhone.test(phone));
            }
            catch (err) {
                console.log('err', err);
                return false;
            }
        }
        if (canReceiveInternationalNumbers) {
            const regexE164 = /^([0-9]{1,3})(\d{2,4})(\d{7,9})$/;
            return Boolean(regexE164.test(phone));
        }
        numbersWithouCodeAreas = Boolean(paramsPhoneValidator?.numberWithoutCodeAreas);
        if (onlyMobilePhoneBR) {
            if (!(phone.length > 4 && phone.length <= (!numbersWithouCodeAreas ? 11 : 9)))
                return false;
            const regexDDD = !numbersWithouCodeAreas ? /(\d{2})$/g : '';
            const phoneFormat = phone
                .split('')
                .reverse()
                .join('')
                .replace(regexDDD, '');
            if (phoneFormat.length === 9 && phoneFormat.match(/(?:[6-9]9)$/))
                return true;
            if (phoneFormat.length === 8 && phoneFormat.match(/[6-9]$/))
                return true;
            return false;
        }
        if (publicUtilityNumberBR) {
            return Boolean(phone.length >= 3 && phone.length <= 4 && phone.match(/^(1(\d){2,3}$)/));
        }
    }
    return Boolean(phone.match(!numbersWithouCodeAreas
        ? /^([1-9]{2})(9{0,1})([2-9]{1}\d{6,7})$/
        : /^(9{0,1})([2-9]{1}\d{6,7})$/));
};

const validators = {
    cpf: cpfValidator,
    cnpj: cnpjValidator,
    email: emailValidator,
    nis: nisValidator,
    phone: phoneValidator,
};

exports.validators = validators;
//# sourceMappingURL=index.ts.map
